@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,7 +22,7 @@
 
 ===============================================================================
 
-Support functions for file I/O.
+**Support functions for file I/O.**
 
 """
 
@@ -43,9 +43,6 @@ from typing import (Any, BinaryIO, Generator, Iterable, IO, List, TextIO,
                     Tuple, Union)
 import zipfile
 
-# noinspection PyCompatibility
-# import regex
-
 log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
 
@@ -58,6 +55,11 @@ UTF8 = "utf8"
 
 @contextmanager
 def smart_open(filename: str, mode: str = 'Ur') -> IO:
+    """
+    Context manager (for use with ``with``) that opens a filename and provides
+    a :class:`IO` object. If the filename is ``'-'``, however, then
+    ``sys.stdin`` is used for reading and ``sys.stdout`` is used for writing.
+    """
     # https://stackoverflow.com/questions/17602878/how-to-handle-both-with-open-and-sys-stdout-nicely  # noqa
     # https://stackoverflow.com/questions/1744989/read-from-file-or-stdin/29824059#29824059  # noqa
     if filename == '-':
@@ -79,24 +81,41 @@ def smart_open(filename: str, mode: str = 'Ur') -> IO:
 # =============================================================================
 
 def writeline_nl(fileobj: TextIO, line: str) -> None:
+    """
+    Writes a line plus a terminating newline to the file.
+    """
     fileobj.write(line + '\n')
 
 
 def writelines_nl(fileobj: TextIO, lines: Iterable[str]) -> None:
-    # Since fileobj.writelines() doesn't add newlines...
-    # http://stackoverflow.com/questions/13730107/writelines-writes-lines-without-newline-just-fills-the-file  # noqa
+    """
+    Writes lines, plus terminating newline characters, to the file.
+
+    (Since :func:`fileobj.writelines` doesn't add newlines...
+    http://stackoverflow.com/questions/13730107/writelines-writes-lines-without-newline-just-fills-the-file)
+    """  # noqa
     fileobj.write('\n'.join(lines) + '\n')
 
 
 def write_text(filename: str, text: str) -> None:
+    """
+    Writes text to a file.
+    """
     with open(filename, 'w') as f:  # type: TextIO
         print(text, file=f)
 
 
 def write_gzipped_text(basefilename: str, text: str) -> None:
-    # Lintian wants non-timestamped gzip files, or it complains:
-    # https://lintian.debian.org/tags/package-contains-timestamped-gzip.html
-    # See http://stackoverflow.com/questions/25728472/python-gzip-omit-the-original-filename-and-timestamp  # noqa
+    """
+    Writes text to a file compressed with ``gzip`` (a ``.gz`` file).
+    The filename is used directly for the "inner" file and the extension
+    ``.gz`` is appended to the "outer" (zipped) file's name.
+    
+    This function exists primarily because Lintian wants non-timestamped gzip
+    files, or it complains:
+    - https://lintian.debian.org/tags/package-contains-timestamped-gzip.html
+    - See http://stackoverflow.com/questions/25728472/python-gzip-omit-the-original-filename-and-timestamp
+    """  # noqa
     zipfilename = basefilename + '.gz'
     compresslevel = 9
     mtime = 0
@@ -111,10 +130,15 @@ def write_gzipped_text(basefilename: str, text: str) -> None:
 # =============================================================================
 
 def get_lines_without_comments(filename: str) -> List[str]:
+    """
+    Reads a file, and returns all lines as a list, left- and right-stripping
+    the lines and removing everything on a line after the first ``#``.
+    NOTE: does not cope well with quoted ``#`` symbols!
+    """
     lines = []
     with open(filename) as f:
         for line in f:
-            line = line.partition('#')[0]
+            line = line.partition('#')[0]  # the part before the first #
             line = line.rstrip()
             line = line.lstrip()
             if line:
@@ -128,6 +152,16 @@ def get_lines_without_comments(filename: str) -> List[str]:
 
 def gen_textfiles_from_filenames(
         filenames: Iterable[str]) -> Generator[TextIO, None, None]:
+    """
+    Generates file-like objects from a list of filenames.
+
+    Args:
+        filenames: iterable of filenames
+
+    Yields:
+        each file as a :class:`TextIO` object
+
+    """
     for filename in filenames:
         with open(filename) as f:
             yield f
@@ -135,12 +169,29 @@ def gen_textfiles_from_filenames(
 
 def gen_lines_from_textfiles(
         files: Iterable[TextIO]) -> Generator[str, None, None]:
+    """
+    Generates lines from file-like objects.
+
+    Args:
+        files: iterable of :class:`TextIO` objects
+
+    Yields:
+        each line of all the files
+
+    """
     for file in files:
         for line in file:
             yield line
 
 
 def gen_lower(x: Iterable[str]) -> Generator[str, None, None]:
+    """
+    Args:
+        x: iterable of strings
+
+    Yields:
+        each string in lower case
+    """
     for string in x:
         yield string.lower()
 
@@ -148,7 +199,18 @@ def gen_lower(x: Iterable[str]) -> Generator[str, None, None]:
 def gen_lines_from_binary_files(
         files: Iterable[BinaryIO],
         encoding: str = UTF8) -> Generator[str, None, None]:
-    # Strips out newlines
+    """
+    Generates lines from binary files.
+    Strips out newlines.
+
+    Args:
+        files: iterable of :class:`BinaryIO` file-like objects
+        encoding: encoding to use
+
+    Yields:
+        each line of all the files
+
+    """
     for file in files:
         for byteline in file:
             line = byteline.decode(encoding).strip()
@@ -159,6 +221,23 @@ def gen_files_from_zipfiles(
         zipfilenames_or_files: Iterable[Union[str, BinaryIO]],
         filespec: str,
         on_disk: bool = False) -> Generator[BinaryIO, None, None]:
+    """
+
+    Args:
+        zipfilenames_or_files: iterable of filenames or :class:`BinaryIO`
+            file-like objects, giving the ``.zip`` files
+        filespec: filespec to filter the "inner" files against
+        on_disk: if ``True``, extracts inner files to disk yields file-like
+            objects that access disk files (and are therefore seekable); if
+            ``False``, extracts them in memory and yields file-like objects to
+            those memory files (which will not be seekable; e.g.
+            https://stackoverflow.com/questions/12821961/)
+
+    Yields:
+        file-like object for each inner file matching ``filespec``; may be
+        in memory or on disk, as per ``on_disk``
+
+    """
     for zipfilename_or_file in zipfilenames_or_files:
         with zipfile.ZipFile(zipfilename_or_file) as zf:
             infolist = zf.infolist()  # type: List[zipfile.ZipInfo]
@@ -183,6 +262,18 @@ def gen_files_from_zipfiles(
 def gen_part_from_line(lines: Iterable[str],
                        part_index: int,
                        splitter: str = None) -> Generator[str, None, None]:
+    """
+    Splits lines with ``splitter`` and yields a specified part by index.
+
+    Args:
+        lines: iterable of strings
+        part_index: index of part to yield
+        splitter: string to split the lines on
+
+    Yields:
+        the specified part for each line
+
+    """
     for line in lines:
         parts = line.split(splitter)
         yield parts[part_index]
@@ -190,6 +281,19 @@ def gen_part_from_line(lines: Iterable[str],
 
 def gen_part_from_iterables(iterables: Iterable[Any],
                             part_index: int) -> Generator[Any, None, None]:
+    """
+    Yields the *n*\ th part of each thing in ``iterables``.
+
+    Args:
+        iterables: iterable of anything
+        part_index: part index
+
+    Yields:
+        ``item[part_index] for item in iterable``
+
+    """
+    # RST: make part of word bold/italic:
+    # https://stackoverflow.com/questions/12771480/part-of-a-word-bold-in-restructuredtext  # noqa
     for iterable in iterables:
         yield iterable[part_index]
 
@@ -199,6 +303,20 @@ def gen_rows_from_csv_binfiles(
         encoding: str = UTF8,
         skip_header: bool = False,
         **csv_reader_kwargs) -> Generator[Iterable[str], None, None]:
+    """
+    Iterate through binary file-like objects that are CSV files in a specified
+    encoding. Yield each row.
+
+    Args:
+        csv_files: iterable of :class:`BinaryIO` objects
+        encoding: encoding to use
+        skip_header: skip the header (first) row of each file?
+        csv_reader_kwargs: arguments to pass to :func:`csv.reader`
+
+    Yields:
+        rows from the files
+
+    """
     dialect = csv_reader_kwargs.pop('dialect', None)
     for csv_file_bin in csv_files:
         # noinspection PyTypeChecker
@@ -224,8 +342,8 @@ def gen_rows_from_csv_binfiles(
 
 def webify_file(srcfilename: str, destfilename: str) -> None:
     """
-    Rewrites a file from "srcfilename" to "destfilename", HTML-escaping it in
-    the process.
+    Rewrites a file from ``srcfilename`` to ``destfilename``, HTML-escaping it
+    in the process.
     """
     with open(srcfilename) as infile, open(destfilename, 'w') as ofile:
         for line_ in infile:
@@ -237,7 +355,7 @@ def remove_gzip_timestamp(filename: str,
                           gzip_executable: str = "gzip",
                           gzip_args: List[str] = None) -> None:
     """
-    Uses external gunzip/gzip tools to remove a gzip timestamp.
+    Uses external ``gunzip``/``gzip`` tools to remove a ``gzip`` timestamp.
     Necessary for Lintian.
     """
     gzip_args = gzip_args or [
@@ -268,6 +386,11 @@ def remove_gzip_timestamp(filename: str,
 def replace_in_file(filename: str, text_from: str, text_to: str) -> None:
     """
     Replaces text in a file.
+
+    Args:
+        filename: filename to process (modifying it in place)
+        text_from: original text to replace
+        text_to: replacement text
     """
     log.info("Amending {}: {} -> {}".format(
         filename, repr(text_from), repr(text_to)))
@@ -282,6 +405,10 @@ def replace_multiple_in_file(filename: str,
                              replacements: List[Tuple[str, str]]) -> None:
     """
     Replaces multiple from/to string pairs within a single file.
+
+    Args:
+        filename: filename to process (modifying it in place)
+        replacements: list of ``(from_text, to_text)`` tuples
     """
     with open(filename) as infile:
         contents = infile.read()
@@ -296,7 +423,13 @@ def replace_multiple_in_file(filename: str,
 def convert_line_endings(filename: str, to_unix: bool = False,
                          to_windows: bool = False) -> None:
     """
-    Converts a file from UNIX -> Windows line endings, or the reverse.
+    Converts a file (in place) from UNIX to Windows line endings, or the
+    reverse.
+
+    Args:
+        filename: filename to modify (in place)
+        to_unix: convert Windows (CR LF) to UNIX (LF)
+        to_windows: convert UNIX (LF) to Windows (CR LF)
     """
     assert to_unix != to_windows
     with open(filename, "rb") as f:
@@ -325,6 +458,10 @@ def convert_line_endings(filename: str, to_unix: bool = False,
 def is_line_in_file(filename: str, line: str) -> bool:
     """
     Detects whether a line is present within a file.
+
+    Args:
+        filename: file to check
+        line: line to search for (as an exact match)
     """
     assert "\n" not in line
     with open(filename, "r") as file:
@@ -336,7 +473,11 @@ def is_line_in_file(filename: str, line: str) -> bool:
 
 def add_line_if_absent(filename: str, line: str) -> None:
     """
-    Adds a line (at the end) if it's not already in the file.
+    Adds a line (at the end) if it's not already in the file somewhere.
+
+    Args:
+        filename: filename to modify (in place)
+        line: line to append (which must not have a newline in)
     """
     assert "\n" not in line
     if not is_line_in_file(filename, line):
